@@ -19,25 +19,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
+#pragma once
 
-#ifndef OSMO_PCAP_SERVER_H
-#define OSMO_PCAP_SERVER_H
-
-#include "wireformat.h"
-#include "osmo_tls.h"
-
-#include <osmocom/core/select.h>
-#include <osmocom/core/linuxlist.h>
-#include <osmocom/core/write_queue.h>
-
+#include <stdbool.h>
+#include <time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
 #include <pcap.h>
 
-#include <stdbool.h>
-#include <time.h>
+#include <osmocom/core/select.h>
+#include <osmocom/core/linuxlist.h>
+#include <osmocom/core/write_queue.h>
+#include <osmocom/core/socket.h>
+
+#include <osmo-pcap/wireformat.h>
+#include <osmo-pcap/osmo_tls.h>
+#include <osmo-pcap/osmo_pcap_file.h>
 
 struct rate_ctr_group;
 struct rate_ctr_group_desc;
@@ -81,16 +80,22 @@ struct osmo_pcap_conn {
 	/* name */
 	char *name;
 	char *remote_host;
-	int no_store;
-	struct in_addr remote_addr;
+	bool store;
+	struct osmo_sockaddr rem_addr;
 
 	/* Remote connection */
 	struct osmo_wqueue rem_wq;
 	int local_fd;
+	/* canonicalized absolute pathname of pcap file we write to */
 	char *curr_filename;
+	/* Current write offset of the file we write to (local_fd) */
+	off_t wr_offset;
 
 	/* pcap stuff */
-	struct pcap_file_header file_hdr;
+	enum osmo_pcap_fmt file_fmt;
+	bool pcapng_endian_swapped;
+	uint8_t *file_hdr;
+	uint32_t file_hdr_len;
 
 	/* last time */
 	struct tm last_write;
@@ -98,8 +103,9 @@ struct osmo_pcap_conn {
 	/* read buffering */
 	int state;
 	int pend;
-	int reopen;
+	bool reopen_delayed;
 	struct osmo_pcap_data *data;
+	size_t data_max_len; /* size of allocated buffer in data->data. */
 
 	/* statistics */
 	struct rate_ctr_group *ctrg;
@@ -111,6 +117,15 @@ struct osmo_pcap_conn {
 	struct osmo_tls_session tls_session;
 };
 
+void osmo_pcap_conn_free(struct osmo_pcap_conn *conn);
+void vty_server_init(void);
+void osmo_pcap_conn_close(struct osmo_pcap_conn *conn);
+int osmo_pcap_conn_process_data(struct osmo_pcap_conn *conn, const uint8_t *data, size_t len);
+void osmo_pcap_conn_restart_trace(struct osmo_pcap_conn *conn);
+void osmo_pcap_conn_close_trace(struct osmo_pcap_conn *conn);
+void osmo_pcap_conn_event(struct osmo_pcap_conn *conn,
+			  const char *event, const char *data);
+
 struct osmo_pcap_server {
 	struct llist_head conn;
 
@@ -139,6 +154,7 @@ struct osmo_pcap_server {
 	bool dh_params_allocated;
 
 	char *base_path;
+	char *completed_path;
 	mode_t permission_mask;
 	off_t max_size;
 	bool max_size_enabled;
@@ -157,14 +173,9 @@ struct osmo_pcap_server {
 extern struct osmo_pcap_server *pcap_server;
 extern const struct rate_ctr_group_desc pcap_peer_group_desc;
 extern const struct rate_ctr_group_desc pcap_server_group_desc;
+struct osmo_pcap_server *osmo_pcap_server_alloc(void *ctx);
 
 void osmo_pcap_server_reopen(struct osmo_pcap_server *server);
 int osmo_pcap_server_listen(struct osmo_pcap_server *server);
-struct osmo_pcap_conn *osmo_pcap_server_find(struct osmo_pcap_server *ser,
+struct osmo_pcap_conn *osmo_pcap_server_find_or_create(struct osmo_pcap_server *ser,
 					     const char *name);
-void osmo_pcap_server_delete(struct osmo_pcap_conn *conn);
-void vty_server_init(void);
-void osmo_pcap_server_close_trace(struct osmo_pcap_conn *conn);
-void osmo_pcap_server_close_conn(struct osmo_pcap_conn *conn);
-
-#endif
